﻿
using System;
using System.Data;
using System.Drawing;
using System.Windows.Forms;

using MindFusion.Spreadsheet;


namespace SpreadsheetDatabase
{
	public partial class Form1 : Form
	{
		public Form1()
		{
			InitializeComponent();

			// Load the data
			loading = true;
			LoadData();
			loading = false;

			// Make sure the filled cells display the default borders, otherwise the
			// cells in the auto-indent columns will have no borders
			workbookView1.DrawFilledCellBorders = true;

			// Hide the hidden columns/rows indicators
			workbookView1.ShowHiddenHeaderIndicators = false;

			// Prevent the user from performing auto-filling
			workbookView1.AllowAutoFill = false;

			// Hide the tab buttons and the formula bar
			workbookView1.ShowTabs = false;
			workbookView1.ShowFormulaBar = false;

			// Handle the necessary events
			workbookView1.InplaceEditStarting += OnInplaceEditStarting;
			workbookView1.SelectionChanged += OnSelectionChanged;
			workbookView1.ActiveCellChanged += OnActiveCellChanged;
			workbook1.WorksheetCellChanging += OnWorksheetCellChanging;
			workbook1.WorksheetCellChanged += OnWorksheetCellChanged;
		}

		private void LoadData()
		{
			// Create a worksheet representing the data
			var data = categoriesTableAdapter1.GetData();
			var sheet = workbook1.Worksheets.Add(nwindDataSet1.Categories.TableName);

			sheet.BeginInit();
			foreach (DataColumn column in data.Columns)
			{
				var sheetColumn = sheet.Columns[column.Ordinal];
				sheetColumn.Title = column.Caption;
				if (column.AutoIncrement)
				{
					// Style auto-increment columns differently. Also tag those columns
					// to prevent them from being edited (see the event handlers below)
					sheetColumn.Tag = true;
					sheetColumn.Style.Background = new MindFusion.Drawing.SolidBrush(Color.FromArgb(251, 246, 221));
				}
			}

			// Prevent the picture field from editing too
			sheet.Columns[3].Tag = false;

			sheet.Tag = data.Columns.Count;
			sheet.Columns[data.Columns.Count, sheet.Columns.Count - 1].IsHidden = true;
			int i = 0;
			foreach (DataRow row in data.Rows)
			{
				foreach (DataColumn column in data.Columns)
					sheet.Cells[column.Ordinal, i].Data = row[column];
				i++;
			}
			sheet.Rows[i + 1, sheet.Rows.Count - 1].IsHidden = true;
			sheet.Rows[i].Title = "";

			workbookView1.ActiveWorksheet = sheet;
			workbookView1.ResizeColumnsToFit(0, data.Columns.Count - 1);
			Measure defaultColumnWidth = "48pt";
			for (int c = 0; c < data.Columns.Count; c++)
			{
				if (sheet.Columns[c].Width.Amount < defaultColumnWidth.Amount)
					sheet.Columns[c].Width = defaultColumnWidth;
			}
			sheet.EndInit();

			UpdateActiveRow();
		}

		private void OnInplaceEditStarting(object sender, InplaceEditValidationEventArgs e)
		{
			// Prevent the auto-increment columns from being edited
			var sheet = workbookView1.ActiveWorksheet;
			var cell = e.Item as Cell;
			if (cell == null)
				return;

			var column = sheet.Columns[cell.Column];
			if (column.Tag is bool)
				e.Cancel = true;
		}

		private void OnSelectionChanged(object sender, EventArgs e)
		{
			if (changingSelection)
				return;

			// Prevent selecting multiple cells
			var selection = workbookView1.Selection;
			if (selection.Left != selection.Right || selection.Top != selection.Bottom)
			{
				changingSelection = true;
				selection.Set(workbookView1.ActiveCell.Column, workbookView1.ActiveCell.Row);
				changingSelection = false;
			}
		}

		private void OnActiveCellChanged(object sender, EventArgs e)
		{
			// Highlight the row containing the selection
			UpdateActiveRow();
		}

		private void UpdateActiveRow()
		{
			int newActiveRow = workbookView1.ActiveCell.Row;
			if (newActiveRow == activeRow)
				return;

			var sheet = workbookView1.ActiveWorksheet;
			if (activeRow != -1)
				sheet.Rows[activeRow].Style.Reset();
			activeRow = newActiveRow;
			if (activeRow != -1)
				sheet.Rows[activeRow].Style.Background = new MindFusion.Drawing.SolidBrush(Color.FromArgb(226, 236, 252));
		}

		private void OnWorksheetCellChanging(object sender, CellValidationEventArgs e)
		{
			if (loading)
				return;

			// Auto-increment columns cannot be edited directly
			var sheet = e.Worksheet;
			var cell = e.Cell;
			var column = sheet.Columns[cell.Column];
			if (column.Tag is bool)
				e.Cancel = true;
		}

		private void OnWorksheetCellChanged(object sender, CellChangedEventArgs e)
		{
			if (loading)
				return;
			if (changingCell)
				return;

			// If the bottommost row is edited, 'add' a new, empty row
			var worksheet = e.Worksheet;
			if (worksheet.Rows[e.Cell.Row + 1].IsHidden)
			{
				worksheet.Rows[e.Cell.Row + 1].IsHidden = false;
				worksheet.Rows[e.Cell.Row].Title = null;
				worksheet.Rows[e.Cell.Row + 1].Title = "";
			}

			// Also make sure to automatically update all auto-increment columns
			bool newRow = false;
			double newRowId = 1;
			int columnCount = (int)worksheet.Tag;
			for (int i = 0; i < columnCount; i++)
			{
				var column = worksheet.Columns[i];
				if (column.Tag is bool && (bool)column.Tag)
				{
					var cell = worksheet.Cells[column.Index, e.Cell.Row];
					if (cell.Data == null)
					{
						if (e.Cell.Row > 0)
						{
							var previous = worksheet.Cells[column.Index, e.Cell.Row - 1];
							if (previous.Value is double)
								newRowId = (double)previous.Value + 1;
						}

						changingCell = true;
						cell.Data = newRowId;
						changingCell = false;

						newRow = true;
					}
				}
			}

			// Update the database with the change
			DataRow tableRow = null;
			if (newRow)
			{
				tableRow = nwindDataSet1.Categories.Rows.Add(
					newRowId, worksheet.Cells[1, e.Cell.Row].Data, worksheet.Cells[2, e.Cell.Row].Data, null);
			}
			else
			{
				tableRow = nwindDataSet1.Categories.Rows[e.Cell.Row];
				tableRow[e.Cell.Column] = e.NewData;
			}

			categoriesTableAdapter1.Update(tableRow);
		}


		private bool loading;
		private bool changingSelection;
		private bool changingCell;
		private int activeRow = -1;
	}
}
